﻿Imports System.Windows.Threading
Imports FuncClasses
Imports System.Net
Imports System.Collections.ObjectModel
Imports HtmlAgilityPack

Public Class DPD
    Implements IPackager

    Private _id As String
    WithEvents uicontrol As New DPDControl

    Public Sub New(ByVal ID As String)
        _id = ID
        If String.IsNullOrEmpty(ID) = False Then
            Try : RefreshState() : Catch ex As Exception : End Try
        End If
    End Sub

    Public Property Package_ID As String Implements IPackager.Package_ID
        Get
            Return _id
        End Get
        Set(value As String)
            _id = value
            Try : RefreshState() : Catch ex As Exception : End Try
        End Set
    End Property

    Public Sub RefreshState() Implements IPackager.RefreshState
        GetAllStates()
    End Sub

    Public ReadOnly Property InfoControl As System.Windows.UIElement Implements IPackager.InfoControl
        Get
            Return uicontrol
        End Get
    End Property

#Region "GetAllStates"
    WithEvents dclient2 As New Net.WebClient()
    WithEvents timer2 As New DispatcherTimer
    Dim details As New ObservableCollection(Of KeyValuePairs)

    Public Sub GetAllStates() Implements IPackager.GetAllStates
        details.Clear()
        infolaststate = Nothing
        dclient2.Encoding = Text.Encoding.UTF8
        dclient2.DownloadStringAsync(New Uri("https://tracking.dpd.de/cgi-bin/delistrack?pknr=" + Package_ID + "&typ=1&lang=de"))
        timer2.Interval = New TimeSpan(0, 0, 0, 0, 5)
        timer2.Start()
    End Sub

    Private Sub CompletedDetails(sender As Object, e As DownloadStringCompletedEventArgs) Handles dclient2.DownloadStringCompleted
        Try
            '### Analyse des Results ###'
            Dim doc As New HtmlDocument
            doc.LoadHtml(e.Result)
            Dim link As HtmlNode
            link = doc.DocumentNode.SelectSingleNode("//table[@id=""plcTable""]") ' Tabelle mit mehr Infos
            For Each item As HtmlNode In link.ChildNodes
                Dim list As New Collection(Of String)
                If item.Name = "tr" AndAlso item.Attributes("class").Value.Contains("table_content_col") Then
                    For Each childitem As HtmlNode In item.ChildNodes
                        If childitem.Name = "td" Then
                            list.Add(childitem.InnerText.Trim.Replace(vbCrLf, "").Replace("&nbsp;", " "))
                        End If
                    Next
                    '### Datum - Depot - Scannung ###'
                    details.Add(New KeyValuePairs(list(0), list(1), list(2)))
                End If
            Next

            '### Produkt - Letzter Status - Empfänger
            infolaststate = details(details.Count - 1)
        Catch ex As Exception : End Try
    End Sub

    Private Sub Timer2_Tick(sender As Object, e As EventArgs) Handles timer2.Tick
        If infolaststate IsNot Nothing Then
            timer2.Stop()
            uicontrol.SetLastState(infolaststate)
            uicontrol.SetDetails(details)
        End If
    End Sub
#End Region

#Region "Letzter Status"
    Dim infolaststate As KeyValuePairs
    Public Sub GetLastState() Implements IPackager.GetLastState
        GetAllStates
    End Sub
#End Region

    Public ReadOnly Property Details1 As ObservableCollection(Of FuncClasses.KeyValuePairs) Implements IPackager.Details
        Get
            Return details
        End Get
    End Property

    Public ReadOnly Property LastState As String Implements IPackager.LastState
        Get
            Return infolaststate.Value
        End Get
    End Property

    Public ReadOnly Property Packager As Packager Implements IPackager.Packager
        Get
            Return PacketTracking.Packager.DPD
        End Get
    End Property

    Private Sub Refresh() Handles uicontrol.RefreshRequested
        GetAllStates()
    End Sub
End Class
